# -*- coding: UTF-8 -*-
from django.core.validators import RegexValidator
from django.db import models
from os.path import basename

from portailuser.models import PortailUser, Ecole
from bbcode.models import BBCodeField


TAILLE_MAX_NOM_FICHIER = 100
DOSSIER_STOCKAGE_FICHIERS = "frappe"


class Annee(models.Model):
    nom = models.CharField(max_length=16)
    ecole = models.ForeignKey(Ecole, editable=False, verbose_name="École", on_delete=models.CASCADE)

    def filieres(self):
        return Filiere.objects.filter(annee=self).order_by("nom")

    def __str__(self):
        return self.nom


class Filiere(models.Model):
    nom = models.CharField(max_length=64, verbose_name="Filière")
    annee = models.ForeignKey(Annee, verbose_name="Année", on_delete=models.CASCADE)

    def matieres(self):
        return Matiere.objects.filter(filiere=self).order_by("nom")

    def __str__(self):
        return f"{self.annee.nom} - {self.nom}"


class Matiere(models.Model):
    nom = models.CharField(max_length=64, verbose_name="Matière")
    filiere = models.ForeignKey(Filiere, verbose_name="Filière", on_delete=models.CASCADE)

    def documents(self):
        return Document.objects.filter(matiere=self).prefetch_related("tags").order_by("-derniere_modif")

    def __str__(self):
        return f"{self.filiere} - {self.nom}"


class Tag(models.Model):
    _validator = RegexValidator("^#([A-Fa-f0-9]{6}|[A-Fa-f0-9]{3})$", "Entrez une couleur HTML valide (#XXXXXX)")

    nom = models.CharField(max_length=64)
    color = models.CharField(verbose_name="Couleur", max_length=7, validators=[_validator])
    ecole = models.ForeignKey(Ecole, editable=False, verbose_name="École", on_delete=models.CASCADE)

    def __str__(self):
        return self.nom


class Document(models.Model):
    nom = models.CharField(max_length=64)
    annee = models.PositiveSmallIntegerField(verbose_name="Année", blank=True, null=True)
    matiere = models.ForeignKey(Matiere, verbose_name="Matière", on_delete=models.CASCADE)
    description = BBCodeField(pack="message", blank=True, null=True)
    derniere_modif = models.DateTimeField(auto_now=True)
    creation = models.DateTimeField(auto_now_add=True)
    tags = models.ManyToManyField(Tag, blank=True)
    auteur = models.ForeignKey(PortailUser, on_delete=models.CASCADE)

    def fichiers(self):
        return DocumentFichier.objects.filter(document=self).order_by("ordre")

    def commentaires(self):
        return Commentaire.objects.filter(document=self).order_by("creation")

    def __str__(self):
        return self.nom


class DocumentFichier(models.Model):
    document = models.ForeignKey(Document, on_delete=models.CASCADE)
    fichier = models.FileField(upload_to=DOSSIER_STOCKAGE_FICHIERS, max_length=TAILLE_MAX_NOM_FICHIER)
    ordre = models.PositiveSmallIntegerField()

    def __str__(self):
        return basename(self.fichier.path)


class Commentaire(models.Model):
    document = models.ForeignKey(Document, on_delete=models.CASCADE)
    auteur = models.ForeignKey(PortailUser, on_delete=models.CASCADE)
    derniere_modif = models.DateTimeField(auto_now=True)
    creation = models.DateTimeField(auto_now_add=True)
    message = BBCodeField(pack="message")
